# -*- coding:utf-8 -*-
import arcpy

class Toolbox(object):
    def __init__(self):
        self.label = "Sort and Calculate Unique Value"
        self.alias = "sortandcalc"

        # List of tool classes associated with this toolbox
        self.tools = [SortCalculateUniqueValue]

class SortCalculateUniqueValue(object):
    def __init__(self):
        self.label = "Sort and Calculate Unique Value"
        self.description = ""
        self.canRunInBackground = True

    def getParameterInfo(self):
        # First parameter
        in_features = arcpy.Parameter(
            displayName="Input Features",
            name="in_features",
            datatype="GPFeatureLayer",
            parameterType="Required",
            direction="Input")

        # Second parameter
        sort_fields = arcpy.GetParameterInfo("Sort_management")[2]
        sort_fields.displayName = "Sort Field(s)"
        sort_fields.parameterDependencies = [in_features.name]

        # Third parameter
        unique_value_field = arcpy.Parameter(
            displayName="Unique Value Field",
            name="unique_value_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")

        unique_value_field.parameterDependencies = [in_features.name]
        unique_value_field.filter.list = ["Short","Long","Float",
                                          "Single","Double","Text"]

        # Fourth parameter
        start_value = arcpy.Parameter(
            displayName="Start Value",
            name="start_value",
            datatype="GPLong",
            parameterType="Required",
            direction="Input")
        start_value.value = 1

        # Fifth parameter
        interval = arcpy.Parameter(
            displayName="Interval",
            name="interval",
            datatype="GPLong",
            parameterType="Required",
            direction="Input")
        interval.value = 1

        out_features = arcpy.Parameter(
            displayName="Output Features",
            name="out_features",
            datatype="GPFeatureLayer",
            parameterType="Derived",
            direction="Output")

        out_features.parameterDependencies = [in_features.name]
        out_features.schema.clone = True

        params = [in_features, sort_fields, unique_value_field,
                  start_value, interval, out_features]
        return params

    def isLicensed(self):
        return True

    def updateParameters(self, parameters):
        return

    def updateMessages(self, parameters):
        """Provide error message if input is a Shapefile"""
        in_features = parameters[0]
        err_input = "Input must be a Geodatabase Feature Class"

        if in_features.valueAsText:
            desc = arcpy.Describe(in_features.valueAsText)
            if not desc.dataElementType == "DEFeatureClass":
                in_features.setErrorMessage(err_input)                
        return

    def execute(self, parameters, messages):
        in_features = parameters[0].valueAsText
        sort_fields = parameters[1].valueAsText
        unique_value_field = parameters[2].valueAsText
        start_value = parameters[3].value
        interval = parameters[4].value
        sortFields = ""
        
        sortFieldsList = sort_fields.split(";")        
        fields = [unique_value_field]
        orderby = "ORDER BY"
        for sortField in sortFieldsList:
            orderby += " " + sortField[0:-6] + ","
            fields.append(sortField.split(" ")[0])                   
        sqlClause = (None, orderby.strip(','))

        with arcpy.da.UpdateCursor(in_features, fields,
                                   sql_clause=sqlClause) as cursor:
            rec = 0
            for row in cursor:
                if rec == 0:
                    rec = start_value
                else:
                    rec = rec + interval
                row[0] = rec
                cursor.updateRow(row)
        return
